#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*#
# regression.tcl                                                           #
#                                                                          #
# Description:                                                             #
#                                                                          #
#     This Tcl/Tk script defines an interface for reporting the results    #
#     of a built-in regression test.  This test consists of running the    #
#     recognizer on every hand-written sample that was used to train it,   #
#     and keeping track of the results.                                    #
#                                                                          #
# Changes:                                                                 #
#                                                                          #
#     09/20/99  arvo    Split off from Train.tcl.                          #
#                                                                          #
#--------------------------------------------------------------------------#
# Copyright (C) 1999, James Arvo                                           #
#                                                                          #
# This program is free software; you can redistribute it and/or modify it  #
# under the terms of the GNU General Public License as published by the    #
# Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      #
#                                                                          #
# This program is distributed in the hope that it will be useful, but      #
# WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    #
# any particular purpose.  See the GNU General Public License for more     #
# details.                                                                 #
#                                                                          #
#~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*~*#

proc RegressionTestBegin { {frame "" } } {
  global test
  set destroy .
  if { $frame == "" } then {
    set frame .test
    if [catch "toplevel $frame -visual best"] then {
      destroy $frame
      toplevel $frame -visual best
      }
    wm title $frame "Regression Test"
    set destroy $frame
    }
  set test(counter) 0
  set test(frame) $frame
  label  $frame.symb -text "" -width 30    -relief sunken -bd 3 -background white
  canvas $frame.prog -width 150 -height 20 -relief sunken -bd 3 -background white
  frame  $frame.stat -relief ridge -borderwidth 3
  button $frame.quit -text Close -command "destroy $destroy"
  pack   $frame.symb -side top -padx 4 -pady 4
  pack   $frame.prog -side top -padx 4 -pady 4
  pack   $frame.stat -side top -padx 4 -pady 4
  pack   $frame.quit -side top -padx 4 -pady 4
  }

proc RegressionTestProgress { name {frac 0} } {
  global test
  set x [expr $frac * 150]
  set f "[expr int( $frac * 100 )]\%"
  $test(frame).symb config -text $name
  $test(frame).prog delete ALL
  $test(frame).prog create rectangle 0 0 $x 40 -fill blue -tag ALL
  $test(frame).prog create text [expr $x - 20] 14 -text $f -fill white -tag ALL
  update
  }

proc RegressionTestError { actual mistaken } {
  global test
  incr test(counter)
  set lab "$test(frame).stat.error$test(counter)"
  set msg " Symbol $actual was mistaken for \"$mistaken\" "
  label $lab -text $msg -relief sunken -borderwidth 3 -background yellow
  pack  $lab -side top -padx 4 -pady 4
  }

proc RegressionTestResults { data } {
  global test
  destroy $test(frame).symb
  destroy $test(frame).prog
  foreach datum $data {
    set name  [lindex $datum 0]
    set value [lindex $datum 1]
    set lab   "$test(frame).stat.$name"
    switch $name {
      "nerr" { set msg "Number of errors: $value" }
      "ntst" { set msg "Number of tests: $value" }
      "rate" { set msg "Recognition rate: $value\%" }
      "ave"  { set msg "Average uncertainty: $value" }
      "max"  { set sym [lindex $datum 2]
               set num [lindex $datum 3]
               set msg "Maximum uncertainty: $value\noccurring at the symbol \"$sym\",\nsample $num" }
      }
    label $lab -text $msg -width 30 -relief flat -borderwidth 0
    pack  $lab -side top -padx 4 -pady 6
    }    
  update
  }

