/***************************************************************************
* FeatureRecognizer.h                                                      *
*                                                                          *
* Class definitions for feature-based symbol recognizer algorithm, which   *
* is a subclass of the Recognizer class.                                   *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       08/15/99  arvo    Removed fixed-length arrays.                     *
*       12/21/97  arvo    General cleanup.                                 *
*       11/20/97  xfan    Initial coding.                                  *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __FEATURE_RECOGNIZER_INCLUDED__
#define __FEATURE_RECOGNIZER_INCLUDED__

#include <util/List.h>       // Used for chaining letter statistics together.
#include <rec/Symbol.h>      // The Symbol class links names to typefaces.
#include <rec/Config.h>      // Basic constants used throughout.
#include <rec/Recognizer.h>  // Base classes for all symbol recognizers.
#include <rec/Features.h>    // Defines feature vector & feature statistics.


// The FeatureRecognizer class implements the Recognizer class 
// using the K-nearest-neighbors algorithm in a high-dimensional feature space.
// This is the central class of the recognizer module.

class FeatureRecognizer : public Recognizer {

    public:
        FeatureRecognizer( const char *path = NULL );
        virtual ~FeatureRecognizer();
        virtual int Recognize( const Multiline &lines, Profile & );
        virtual int NumSymbols() const { return num_symbols; }
        void SetProgressMeter( progress_meter fun ) { prog_meter = fun; }
		// R.Zanibbi April 29, 2004
		char* GetSymbolNames();

    protected:
        const FeatureStatsVector &GetStats( Symbol * ) const;
        int LoadFeatureMatrices( const char *path );
        int NumFeatures() const { return num_features; }
        int GetExemplar( const char *file, Multiline &, char *name = NULL ) const;

    private:
        FeatureStatsVector *ComputeFeatureDistances( const Multiline & );
        int GetBestCandidates( int top_n, FeatureStatsVector *Index[] );
        int LoadFeatureMatrix( const char *path );
        int num_symbols; 
        int num_features;
        void Progress( int flag, const char *msg );
        progress_meter prog_meter;
        List feature_stats_list;  // To hold instances of FeatureStatsVector
    };

#endif








