/***************************************************************************
* Name.h                                                                   *
*                                                                          *
* Defines "NameTableItem", which is the base class for objects that can be *
* inserted into a "NameTable" object.                                      *
*                                                                          *
*   Changes:                                                               *
*                                                                          *
*       08/06/99    arvo      Added secondary qualifiers to names.         *
*       08/10/98    arvo      Moved from font directory to util.           *
*       06/06/98    arvo      Initial coding.                              *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef __NAME_INCLUDED__
#define __NAME_INCLUDED__

#include <util/Char.h>

class NameTableItem;
class NameTable;

static const int NameHashTableSize = 317;  // Buckets in the hash table.

class NameTableItem {

    public:
        NameTableItem( const char *name, const char *qual = NULL );
        virtual ~NameTableItem() { delete[] name; delete[] qual; }
        const char *Name() const { return name; }
        const char *Qual() const { return qual; }

    private:
        char *name;           // The primary key.
        char *qual;           // A secondary qualifier (optional).
        long  code;           // The actual hash code (before mod).
        NameTableItem *next;  // The next item in hash table bucket.
        NameTableItem *back;  // The previous item in hash table bucket.
        NameTable     *owner; // The name table itself.

    friend class NameTable;
    };


class NameTable {

    public:
        NameTable();
        virtual ~NameTable();
        NameTableItem *Lookup( const char *, const char * = NULL ) const;
        void Clear();
        NameTableItem *operator>>( NameTableItem * ); // Remove an item from table.
        NameTable     &operator<<( NameTableItem * ); // Insert an item into table.
        int  NumEntries() const { return numentries; }
        void DeleteAll();
        NameTableItem *First() const;
        NameTableItem *Next( const NameTableItem * ) const;

    private:
        int numentries;
        NameTableItem *table[ NameHashTableSize ];
    };

#endif








