/***************************************************************************
* test_recognizer.C                                                        *
*                                                                          *
* This is a trivial test program that demonstrates how to invoke the       *
* recognizer. Once you cd to the src/rec directory, you can build it       *
* like this:                                                               *
*                                                                          *
*     CC test_recognizer.C -I.. -L. -lRec -lm -o test_recognizer           *
*                                                                          *
* You can also use "gmake test_recognizer" to build it.  When you run it,  *
* give it the path to your training file directory on the command line.    *
* If none is given, it defaults to the current working directory.          *
*                                                                          *
*--------------------------------------------------------------------------*
* Copyright (C) 1999, James Arvo                                           *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include <cstdlib>
#include <rec/FeatureRecognizer.h>

// The following coordinates are from an actual hand-written "3".  They are 
// encoded  this way to demonstrate the interface; this way, the test program
// needn't bother with pen-input.

float points[][2] = {
  {103,  97}, {104,  95}, {106,  92}, {112,  88},
  {120,  85}, {130,  83}, {138,  84}, {145,  89},
  {149,  97}, {148, 107}, {145, 119}, {139, 129},
  {131, 136}, {122, 143}, {120, 144}, {117, 145},
  {119, 145}, {127, 146}, {134, 149}, {141, 153},
  {148, 160}, {153, 168}, {155, 177}, {156, 185},
  {154, 190}, {148, 195}, {135, 198}, {121, 196},
  {104, 194}, { 94, 191}, { 91, 190}, { 92, 187}
  };

int main( int argc, char *argv[] )
    {
    const char *path = ( argc > 1 ) ? argv[1] : ".";

    FeatureRecognizer rec( path );  // This is the feature-based recognizer.
    Profile           profile;      // The "profile" records the results.
    Multiline         strokes;      // The required encoding for the coordinates.

    // The constructor for "FeatureRecognizer" reads all the
    // feature matrices in the given directory or any of its
    // sub-directories.  Print out how many symbols were 
    // found encoded in those files.

    cout << endl
         << "The recognizer has loaded " 
         << rec.NumSymbols() 
         << " symbols." 
         << endl;


    // Load the coordinates of a hand-drawn stroke into a single
    // Multiline for the recognizer.  The multiline class also accommodates 
    // multi-stroke characters, like "X" and "A".  The coordinates stored 
    // in the "points" array encode a hand-drawn numeral "3".

    for( int i = 0; i < sizeof(points) / (2 * sizeof(float)); i++ )
      strokes << Vec2( points[i][0], points[i][1] );


    // Here's where we actually call the recognizer.  The answer
    // is passed back in the "profile", which contains the best guesses,
    // sorted by decreasing confidence.

    rec.Recognize( strokes, profile );


    // Print out what we found...

    cout << endl;

    cout << "The recognizer offers " << profile.NumChoices() 
         << " choices for the given strokes." 
         << endl;

    cout << "The most likely symbol is \"" 
         << profile.BestSymbol()->Name() 
         << "\", which has a confidence level of " 
         << profile.BestProb() 
         << endl;

    cout << endl;


    // Dump the entire profile using its built-in print method.
    // This allows us to see all the other possibilities.

    cout << "The entire profile looks like this...\n\n";

    cout << profile;

    return EXIT_SUCCESS;
    }


