%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% AlignSymbols.Txl
%	- one pass DRACULAE BST analysis and BB morphing.
%	  Note: we assume that the origin is at the Top Left.
%
% Copyright (C) 2000,2002,2004 Richard Zanibbi
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%
% You may contact the authors by email at zanibbi@cs.queensu.ca,
% or by mail at: Richard Zanibbi, Dept. Computing Science,
% Queen's University, Kingston, Ontario, CANADA, K7L 3N6
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Revision History:
% v. 1.0.0 November 14, 2000
% v. 1.0.1 Mar 1, 2001
%	- Cleaned this up to use new pass structure.
% v. 1.0.2 March 19, 2001
%	-  Added new "keepOnCanvas" method.
%	   (also checks right/bottom now).
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Morphing and Grammar:
include "Grammars/BST.Grammar"
include "Grammars/Symbol_Classes.Grammar"

% Layout pass.
include "Rules/Layout_Analysis.Rules"
% Morphing pass and related routines.
include "Rules/PartialMorph.Rules"
include "Rules/CharacterResizing.Rules"
include "Rules/Packing.Rules"

% Origin moving code.
include "Rules/Files.Rules"
include "Rules/MoveOrigin.Rules"

% Handle command line options.
include "Rules/CommandLineArgs.Rules"

function main
	replace [program]
		I. Baseline Structure Tree
		TheBST[baselineStructureTree]
		_[repeat labelled_bst]

	  %  TheBST[baselineStructureTree]		

	import TXLargs[repeat stringlit]

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Default global settings (configurable from command line)
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	construct ThresholdRatio[number]
		_[+ 1][/ 6]

	construct CentroidRatio[number]
		_[+ 1][/ 4]

	construct OriginLocation[stringlit]
		"Top Left"

	construct Version [stringlit]
		"0.2"

	construct HelpRequested[stringlit]
		"False"

	export OriginLocation
	export Version
	export HelpRequested
	export ThresholdRatio
	export CentroidRatio

	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	% Process command line arguments
	%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
	construct Args[repeat stringlit]
		TXLargs[processArguments]
		#ifdef DEBUG
		[showArgs]  %for debugging.
		#endif

	% Re-import these values; they may have changed.
	import OriginLocation
	import HelpRequested

	where 
		HelpRequested[= "False"]

	construct MaxY[integernumber]
		_[findMaxY TheBST]

	by
	    TheBST
		 [moveOrigin MaxY] % Move origin if necessary.
	     [formatBaselines] % format the BST.	    
 		 [unmark]

	     [moveOrigin MaxY]	% Move origin back if necessary.
	     [keepOnCanvas]		% Keep things on the canvas...
	   	 [linearizeSymbolList] % Make a list of symbols...
end function




