%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% GetSemantics.Txl
%	- txl program to obtain an operator tree from a lexed BST
%
% Copyright (C) 2000,2002,2004 Richard Zanibbi
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; either version 2
% of the License, or (at your option) any later version.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% You should have received a copy of the GNU General Public License
% along with this program; if not, write to the Free Software
% Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
%
% You may contact the authors by email at zanibbi@cs.queensu.ca,
% or by mail at: Richard Zanibbi, Dept. Computing Science,
% Queen's University, Kingston, Ontario, CANADA, K7L 3N6
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Revision History
% v 1.0.0 Mar 1, 2001
% v 1.0.1 April 23, 2004
%	- R. Zanibbi: altered the code so that "GetLexedBST" is no longer
%	  needed to filter the intermediate output of "GetTeX."
%	- Added the routine "unquoteNumbers" to remove quotes around numerals.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Hack: override the BST Grammar...
include "Grammars/BST.Grammar"
include "Grammars/Symbol_Classes.Grammar"

include "Grammars/Expression.Grammar"
include "Rules/Semantics.Rules"

%--------------------------------------------------
% function main
% 	replace [program]
% 	    P[program]
% 	by
% 	    P[semanticAnalysis]
% end function
%-------------------------------------------------- 

function main
	replace [program]
		InputFile[bst_file]

	deconstruct InputFile
		BST1[labelled_bst]
		BST2[labelled_bst]
		'III. Relation Lexed BST
		LexedBST[baselineStructureTree]

	construct Dud[expression]
		"DECIMAL_NUMBER" "2" < (221, 41), (307, 163) >

	% Here we force parsing into an expression.
	% This insures the input is well-formed (according to our grammar).
	%
	% The program will return an error if this fails; this is the
	% desired behaviour.
	construct ParseTree[expression]
		Dud[reparse LexedBST]

	% Cast parsed expression back into "program" type to
	% insure all rewrites are applied.
	construct PromotedParse[program]
		ParseTree

	by
		% Output the operator tree.
		PromotedParse[semanticAnalysis][unquoteNumbers]
end function

% R. Zanibbi, April 26, 2004
% added this function to remove the annoying quotes around numerals.
rule unquoteNumbers
	replace $ [decimal_integernumber]
		Symbol[stringlit]

	construct Dud[number]
		'1

	construct UnquotedSymbol[number]
		Dud[unquote Symbol]

	by
		UnquotedSymbol
end rule

