/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/

// Tcl/Tk includes
#include <tcl.h>
#include <tk.h>
#include "MapWindow.h"

// Tk user commands to map and unmap a window.  For some reason this
// is not provided in the tk user language.
int UnmapWindow(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     const char *argv[])
{
  if (argc != 2)
	{
	fprintf(stderr,"Usage:  UnmapWindow pathname\n");
	return(TCL_ERROR);
	}
  Tk_Window tkwin = Tk_NameToWindow( interp, argv[1], Tk_MainWindow(interp));
  if (tkwin == NULL)
        return( TCL_ERROR);

  Tk_UnmapWindow( tkwin );
  return( TCL_OK );
}

int MapWindow(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     const char *argv[])
{
  if (argc != 2)
	{
	fprintf(stderr,"Usage:  MapWindow pathname\n");
	return(TCL_ERROR);
	}
  Tk_Window tkwin = Tk_NameToWindow( interp, argv[1], Tk_MainWindow(interp));
  if (tkwin == NULL)
	return( TCL_ERROR);

  Tk_MapWindow( tkwin );
  return( TCL_OK );
}


int MapWindow_Init( Tcl_Interp *interp )
{
  if (interp == NULL)
    return TCL_ERROR;

  Tcl_CreateCommand( interp, "MapWindow", MapWindow, NULL, NULL );
  Tcl_CreateCommand( interp, "UnmapWindow", UnmapWindow, NULL, NULL );
  return( TCL_OK );
}
