/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
// Combo generating code
// Thanks to Jays Mackie for helping me with the thinking on this one.

#include <stdio.h>
#include <stdlib.h>
#include "combos.h"
#include "common.h"

#define DEB 0

// Initialise Combos data structure
void Init_Seq(Combos *Com, int N)
{
  int NumberThereWillBe;

  NumberThereWillBe = 1 << N;  // takes into account all lengths <= N, not just N. (just N uses N-1)
//   for (int i=0;i<=N;i++) {
//     NumberThereWillBe += CombinatorialFunction(N,i); // Add in C^{N}_{i}
//   }

  if (DEB) printf("N=%d -> NTWB=%d\n",N,NumberThereWillBe);

  Com->Sequence = (ComboSequence **)calloc(NumberThereWillBe,sizeof(ComboSequence *));
  for (int i=0;i<NumberThereWillBe;i++) {
    Com->Sequence[i] = (ComboSequence *)calloc(N, sizeof(ComboSequence));
  }

  Com->Number = 0;
}

// Check to see if two sequences are the same
int Sequence_Equal(ComboSequence *S1, ComboSequence *S2)
{
  int same;
  int p;
  p = 0;
  same = 1;

  do {
    same = ( S1[p] == S2[p] );
  } while ( (same) && (S2[p++] != -1) ); // check that it does work the way I expect! (WOMBLE!)

  return same;
}

// Copy a sequences
void Copy_Seq(ComboSequence *To, ComboSequence *From)
{
    int p;

    p = 0;
    while (From[p] != -1) {
      To[p] = From[p];
      p++;
    } 
    To[p] = From[p]; // copy the -1 as well
}

// Pop sequence from a combo, removing it from the combo
void Pop_Seq(Combos *Com, ComboSequence *Ret)
{
  Com->Number--;
  Copy_Seq(Ret, Com->Sequence[Com->Number]);
}

// Get specific sequence from a combo, leaving it there
void Get_Seq(Combos *Com, ComboSequence *Ret, int N)
{
  Copy_Seq(Ret, Com->Sequence[N]);
}

// Add to sequence if it isn't already there.
// Return: 1 = was new
//         0 = was already there
int Add_To_Combos(Combos *Com, ComboSequence *ToAdd)
{
  for (int i=0;i<Com->Number;i++) {
    if (Sequence_Equal(Com->Sequence[i], ToAdd)) {
      return 0;
    }
  }
  
 
  Copy_Seq(Com->Sequence[Com->Number], ToAdd);
  Com->Number++;

  return 1;
}

// Are there any sequences here?
int Combo_Size(Combos *Com)
{
  return (Com->Number);
}

void Print_Seq(ComboSequence *S)
{
  // it is important that this function does not print a trailing space
  int p = 0;
  while (S[p] != -1) {
    if (p>0) putchar(' ');
    printf("%d",S[p++]);
  }
}

void Print_Combos(Combos *C)
{
  for (int i=0;i<C->Number;i++) {
    Print_Seq(C->Sequence[i]);
    printf("\n");
  }
}

// Length of sequence
int Seq_Length(ComboSequence *S)
{
  int p=0;

  while (S[p] != -1) p++;

  return p;
}

void Generate_Main(int N, Combos *Generated, int Maximum)
{
  Combos Stack;
  ComboSequence *CurrentSeq;
  ComboSequence *NewSeq;
  int NSP;
  int L;
  bool This_Seq_OK;

  CurrentSeq = (ComboSequence *)calloc(N+1,sizeof(int));
  NewSeq = (ComboSequence *)calloc(N+1,sizeof(int));

  Init_Seq(&Stack, N);

  for (int i=0;i<N;i++) {
    CurrentSeq[i] = 1;
  }
  CurrentSeq[N] = -1; // end of sequence marker

  Add_To_Combos(&Stack, CurrentSeq);
  Add_To_Combos(Generated, CurrentSeq);

  while (Combo_Size(&Stack) > 0) {
    Pop_Seq(&Stack, CurrentSeq); // pull item off stack

    L = Seq_Length(CurrentSeq);
    for (int i=0;i<L-1;i++) { // position in old sequence of first of pairing
      NSP = 0;
      This_Seq_OK = true;
      for (int j=0;j<L;j++) {
	if (j==i) { // first of pair
	    NewSeq[NSP++] = CurrentSeq[j] + CurrentSeq[j+1];
	    if  ((CurrentSeq[j] + CurrentSeq[j+1]) > Maximum) This_Seq_OK = false;
	} else {
	  if (j!=(i+1)) { // isn't second of pair
	    NewSeq[NSP++] = CurrentSeq[j];
	  }
	}
      }
      NewSeq[L-1] = -1; // end of sequence marker
      if (This_Seq_OK) {
	if (Add_To_Combos(Generated, NewSeq)) {
	  Add_To_Combos(&Stack, NewSeq);
	}
      }
    }
  }

}

// Generate those combinations
// N = Total Sum desired
void Generate(int N, 
	      Combos *Generated, 
	      int Maximum)
{
  Init_Seq(Generated, N);
  
  for (int i=1;i<=N;i++) {
    Generate_Main(i, Generated, Maximum);
  }
}


