/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
* Revision History:                                                        *
*  v 1.0 Original Version Steve Smithies (1999)                            *
*  v 1.0.1 R. Zanibbi March 15, 2002                                       *
*    - changed Font_File_Base_Directory to return FFES directory.          *
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include "common.h"

long Factorial(int N)
{
  int res;


  if (N<0) {
    fprintf(stderr,"Factorial in combos.cc only intended for use with N>=0.\nWas called with N=%d\n",N);
    exit(1);
  }

  if (N==0) {
    return 1;
  }

  res = 1;
  while (N>1) {
    res *= N--;
  }
  return res;
}

int CombinatorialFunction(int N,int R)
{
  return( Factorial(N) / ( (Factorial(R)*(Factorial(N-R)) ) ) );
}

// ////////////////////////////////////////////////////////////////////////////////////////

double *Calloc_double(int x)
{
  double *ret;

  ret = (double *)calloc(x, sizeof(double));
    if (ret == NULL) {
      fprintf(stderr,"common:Calloc_double - Couldn't allocate %d doubles\n",x);
      exit(1);
    }
   
    return ret;
}

void Free_double(double *Place)
{
  free(Place);
}


// Alloc a double "array" giving (sort of) double bob[y][x]
double **Calloc_2D_double(int y, int x)
{
  double **ret;

  ret = (double **)calloc(y, sizeof(double *));
  if (ret == NULL) {
    fprintf(stderr,"common:Calloc_2D_double - Couldn't allocate y-part of [%d][%d]\n",y,x);
    exit(1);
  }
  for (int i=0;i<y;i++) {
    ret[i] = Calloc_double(x);
  }

  return ret;
}

// Free a 2D double allocated by the previous routine
void Free_2D_double(double **Place, int y)
{
  for (int i=0;i<y;i++) {
    Free_double(Place[i]);
  }

  free(Place);
}

// ////////////////////////////////////////////////////////////////////////////////////////

char *Calloc_char(int x)
{
  char *ret;

  ret = (char *)calloc(x, sizeof(char));
    if (ret == NULL) {
      fprintf(stderr,"common:Calloc_char - Couldn't allocate %d chars\n",x);
      exit(1);
    }
   
    return ret;
}

void Free_char(char *Place)
{
  free(Place);
}

// Alloc a char "array" giving (sort of) char bob[y][x]
char **Calloc_2D_char(int y, int x)
{
  char **ret;

  ret = (char **)calloc(y, sizeof(char *));
  if (ret == NULL) {
    fprintf(stderr,"common:Calloc_2D_char - Couldn't allocate y-part of [%d][%d]\n",y,x);
    exit(1);
  }

  for (int i=0;i<y;i++) {
    ret[i] = Calloc_char(x);
  }

  return ret;
}

// Free a 2D char allocated by the previous routine
void Free_2D_char(char **Place, int y)
{
  for (int i=0;i<y;i++) {
    Free_char(Place[i]);
  }

  free(Place);
}

// ////////////////////////////////////////////////////////////////////////////////////////

int *Calloc_int(int x)
{
  int *ret;

  ret = (int *)calloc(x, sizeof(int));
    if (ret == NULL) {
      fprintf(stderr,"common:Calloc_int - Couldn't allocate %d ints\n",x);
      exit(1);
    }
   
    return ret;
}

void Free_int(int *Place)
{
  free(Place);
}

// Alloc a int "array" giving (sort of) int bob[y][x]
int **Calloc_2D_int(int y, int x)
{
  int **ret;

  ret = (int **)calloc(y, sizeof(int *));
  if (ret == NULL) {
    fprintf(stderr,"common:Calloc_2D_int - Couldn't allocate y-part of [%d][%d]\n",y,x);
    exit(1);
  }

  for (int i=0;i<y;i++) {
    ret[i] = Calloc_int(x);
  }

  return ret;
}

// Free a 2D int allocated by the previous routine
void Free_2D_int(int **Place, int y)
{

  for (int i=0;i<y;i++) {
    Free_int(Place[i]);
  }

  free(Place);
}


// Alloc a int "array" giving (sort of) int bob[z][y][x]
int ***Calloc_3D_int(int z, int y, int x)
{
  int ***ret;

  ret = (int ***)calloc(z, sizeof(int **));
  if (ret == NULL) {
    fprintf(stderr,"common:Calloc_3D_int - Couldn't allocate z-part of [%d][%d][%d]\n",z,y,x);
    exit(1);
  }

  for (int i=0;i<z;i++) {
    ret[i] = Calloc_2D_int(y,x);
  }

  return ret;
}

void Free_3D_int(int ***Place, int z, int y)
{
  for (int i=0;i<z;i++) {
    Free_2D_int(Place[i],y);
  }
  
  free(Place);
}

// return where the font files' tree is based, using the
// environment variable "FONTFILEDIR"
char *Font_File_Base_Directory(void)
{
  // R. Zanibbi, March 15, 2002: changed FONTFILEDIR to FFES_DIR
  // so environment path gives FFES directory, not path to data dir.
  char *res = getenv("FFES_DIR");
  if (res == NULL)
  {
	printf("Error: FFES_DIR environment variable is undefined.\nFFES_DIR should be defined as the path to the ffes_0.3 directory.\n");
	exit(1);
  }
  else 
    return(res);
}

void Wait_For_Key()
{
  char temp[255]; 
  fflush(stdin);
  printf("UG? "); 
  fflush(stdout); 
  fgets(temp,255,stdin);
}
