/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
*   HISTORY:                                                               *
*     Original Version, 1999                                               *
*     March 4, 2002, Richard Zanibbi                                       *
*       -commented out includes	of latex_preview.h and                     *
*        intersect.h; these are unused, and have been removed              *
*        from the distribution; Do_LaTeX_Preview function                  *
*        has also been removed.                                            *
*       -thumb.h and corresponding thumbnail code has                      *
*        also been commented out and removed.                              *
*     March 15, 2002, Richard Zanibbi                                      *
*       - changed my_init to use an environment variable to set the        *
*         base path for FFES again.  Also removed warning about            *
*         FONTFILEDIR in main (now using FFES_DIR: see common.cc)          *
*     April 23, 2004, Richard Zanibbi                                      *
*     	- changed path to FFES Tk interface file.                          *
*     	- added code to tell the user if FFES_DIR is incorrect.            *
*     April 29, 2004, R. Zanibbi                                           *
*       - added functions to report the number and the names of CIT symbols*
*	  May 21, 2004 R. Zanibbi                                              *
*		- changes to last arg of callbacks (now "const"), and resulting    *
*         new variables in the "strtok" calls in "recognise_this."         *
***************************************************************************/

/******************************************************************************
 * RESEARCH PUBLICATIONS ABOUT FFES:
 
	1. Steve Smithies. Freehand Formula Entry System. Master's thesis, 
	   University of Otago, Dunedin, New Zealand, May 1999. 
	   
	2. S. Smithies, K. Novins, J. Arvo. "A handwriting-based equation editor," 
	   In Proc. Graphics Interface, Kingston, Ontario, Canada, June 1999. 
	   
	3. S. Smithies, K. Novins, and J. Arvo. Equation entry and editing via 
	   handwriting and gesture recognition. Behaviour and 
	   Information Technology, 20(1):53-67, 2001. 

	4. Richard Zanibbi, Kevin Novins, James Arvo and Katherine Zanibbi. 
	   ``Aiding Manipulation of Handwritten Mathematical Expressions through 
	   Style-Preserving Morphs'', In Proceedings of Graphics Interface 2001, 
	   Ottawa, Canada, June 2001, pp. 127-134. 

******************************************************************************/


// Steve's character recogniser!

#include <cstring>
#include <cstdlib>
#include <cstdio>
#include <cassert>
#include <pthread.h>

#include <iostream>
#include <fstream>

// Tcl/Tk includes
#include <tcl.h>
#include <tk.h>

// Smartboard includes
#include <util/Vec2.h>		// 2-D vectors
#include <util/Multiline.h>	// multilines
#include <rec/Recognizer.h>	// the stroke recogniser
#include <util/Box2.h>		// 2-D boxes.

// My includes
#include "charrec.h"
#include "MapWindow.h"

// Global variables.
static CharRec *CR = 0;
char MainTkInterfaceFile[256] = "";

int start_recogniser(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     const char *argv[])
{
  if (CR)
    delete CR;

  CR = new CharRec();

  return TCL_OK;
}

int close_recogniser(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     const char *argv[])
{
  delete CR;
  CR = 0;

  return TCL_OK;
}

int recognise_this(ClientData client_data,
		     Tcl_Interp* interp,
		     int argc,
		     const char *argv[])
{
  if (argc != 5)
    {
      sprintf(interp->result,"Wrong number of arguments in recognise_this in ffes.cc");
      return TCL_ERROR;
  }

  int NumberOfMultilines = atoi(argv[1]);
  int *OriginalCollection = new int[NumberOfMultilines];
    int *PointCounts = new int[NumberOfMultilines];

  char *p;
  int c = 0;
  // R. Zanibbi, May 21, 2004: use of const for Tcl/Tk8.4 callbacks
  // means that the "strtok" calls (this one and the next two) must
  // be altered so that a variable containing the *value* of the passed
  // arguments is used, rather than the arguments themselves.
  char t1[strlen(argv[2])+1];
  t1[0] = '\0';
  strcpy(t1,argv[2]);
  p = strtok(t1, " ");
  while (p)
    {
      OriginalCollection[c++] = atoi(p);
      p = strtok(0, " ");
    }
  assert(c == NumberOfMultilines);


  int TotalPoints = 0;
  c = 0;
  char t2[strlen(argv[3])+1];
  t2[0] = '\0';
  strcpy(t2,argv[3]);
  p = strtok(t2, " ");
  while (p)
    {
      PointCounts[c] = atoi(p);
      TotalPoints += PointCounts[c];
      c++;
      p = strtok(0, " ");
    }

  assert(c == NumberOfMultilines);

  int *Points = new int[TotalPoints * 2];

  c = 0;
  char t3[strlen(argv[4])+1];
  t3[0] = '\0';
  strcpy(t3,argv[4]);
  p = strtok(t3, " ");
  while (p)
    {
      Points[c++] = atoi(p);
      p = strtok(0, " ");
    }
  assert( c == (TotalPoints * 2) );

  CR->DoNormalRec(NumberOfMultilines,
		  OriginalCollection,
		  PointCounts,
		  Points);

  delete PointCounts;
  delete Points;

  char res[2048];
  CR->GetResultForTclTk(res);
  sprintf(interp->result,res);

  return TCL_OK;
}

int boxes_overlap(ClientData client_data,
		  Tcl_Interp* interp,
		  int argc,
		  const char *argv[])
{
  char res[1024];
  if (argc!=9) {
    sprintf(interp->result,"Wrong number of arguments in boxes_overlap in ffes.cc");
    return TCL_ERROR;
  }

  Vec2 tl1(atoi(argv[1]),atoi(argv[2]));
  Vec2 br1(atoi(argv[3]),atoi(argv[4]));
  Vec2 tl2(atoi(argv[5]),atoi(argv[6]));
  Vec2 br2(atoi(argv[7]),atoi(argv[8]));

  Box2d Box1(tl1,br1);
  Box2d Box2(tl2,br2); 
  
  sprintf(res,"%d",Box1.dist(Box2)==0?1:0);
  Tcl_SetResult(interp, res, TCL_VOLATILE);

  return TCL_OK;
}

int LineIntersectTest(ClientData client_data,
		      Tcl_Interp* interp,
		      int argc,
		      const char *argv[])
{
  if (argc!=9) {
    sprintf( interp->result, "wrong number of arguments for ffes.cc:LineIntersectTest (want AxAy,BxBy,CxCy,DxDy)");
    return TCL_ERROR;
  }

  Position_2D A;
  Position_2D B;
  Position_2D C;
  Position_2D D;

  A.x = atoi(argv[1]);
  A.y = atoi(argv[2]);
  B.x = atoi(argv[3]);
  B.y = atoi(argv[4]);
  C.x = atoi(argv[5]);
  C.y = atoi(argv[6]);
  D.x = atoi(argv[7]);
  D.y = atoi(argv[8]);
  
  int res = LineIntersectTest(A,B,C,D);
  char tclres[1024];
  
  sprintf(tclres,"%d",res?1:0);
  Tcl_SetResult(interp, tclres, TCL_VOLATILE);
  return TCL_OK;
}

// R. Zanibbi, April 29, 2004: new Tcl command to get the number of symbols.
int number_CIT_symbols(ClientData client_data,
		      Tcl_Interp* interp,
		      int argc,
		      const char *argv[])
{
	int NumClasses = CR-> NumberClasses();
	char result[8];
	sprintf(result,"%d",NumClasses);
 	Tcl_SetResult(interp, result, TCL_VOLATILE);
 	return TCL_OK;
}

// R. Zanibbi, April 29, 2003: another new Tcl command to return a string
// containing the list of available CIT symbol classes.
int CIT_symbols(ClientData client_data,
		      Tcl_Interp* interp,
		      int argc,
		      const char *argv[])
{
	char* pcNames = NULL;
	pcNames = CR -> GetSymbolNames();
 	Tcl_SetResult(interp, pcNames, TCL_VOLATILE);
	delete pcNames; // "new" is used in GetSymbolNames()
 	return TCL_OK;
}



int my_init( Tcl_Interp *interp )
{
  if (Tcl_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
  if (Tk_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }

  Tcl_CreateCommand( interp, "start_recogniser", start_recogniser, NULL, NULL);
  Tcl_CreateCommand( interp, "close_recogniser", close_recogniser, NULL, NULL);
  Tcl_CreateCommand( interp, "recognise_this", recognise_this, NULL, NULL);

  Tcl_CreateCommand( interp, "Boxes_Overlap", boxes_overlap, NULL, NULL);
  Tcl_CreateCommand( interp, "LineIntersectTest",           LineIntersectTest,           NULL, NULL);

  // R. Zanibbi, April 29, 2004
  // Adding function to return the number of CIT symbol classes in Tcl.
  Tcl_CreateCommand( interp, "NumberSymbolClasses", number_CIT_symbols, NULL, NULL);
  Tcl_CreateCommand( interp, "CIT_SymbolList", CIT_symbols, NULL, NULL);

  if (MapWindow_Init(interp) == TCL_ERROR) {
    return TCL_ERROR;
  }
 
  // R. Zanibbi, March 15, 2002: use an environment variable so
  // that we can run FFES places other than the ffes_0.2 directory.
  //
  // R. Zanibbi April 23, 2004: changed path to interface file.
  char * FFESDir = getenv("FFES_DIR");
  char * InterfaceFile = "/FFES_0.3/src/Tk_Interface/Interface.tk";
  
  if (FFESDir == NULL) {
	  cerr << "ERROR: FFES_DIR needs to be defined in the environment.\n\tIt"
		  << " should point to the base directory of FFES (e.g. <path>/ffes_0.3)\n";
	  exit(1);
  }

  strcat(MainTkInterfaceFile,FFESDir);
  strcat(MainTkInterfaceFile,InterfaceFile);

  // R. Zanibbi, April 24, 2003: adding explicit test for FFES_DIR
  // value, telling user if they need to make a correction.
  ifstream InterfaceExists(MainTkInterfaceFile);
  if(! InterfaceExists)
  {
	  cerr << "ERROR: Could not find the main interface file (Interface.tk) for FFES at:\n"
		   << "    $(FFES_DIR)" << InterfaceFile << "\n\n"
		   << "    Perhaps the FFES_DIR environment variable is incorrectly defined.\n"
		   << "    Currently FFES_DIR is defined as: \n"
		   << "	   " << FFESDir << "\n\n"
		   << "    Please make sure that FFES_DIR points to the base directory of FFES.\n"
		   << "    (e.g. <path>/ffes_0.3)\n";
	  	
	  InterfaceExists.close();
	  exit(1);
  }
  
  InterfaceExists.close();

  Tcl_SetVar( interp, "tcl_rcFileName", MainTkInterfaceFile, TCL_GLOBAL_ONLY );
  return 0;
}


int main(int argc, char **argv)
{
  
  Tk_Main( argc, argv, my_init );
  return 0;
}
