/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <assert.h>

#include "intlist.h"

IntList::IntList()		// Contructor - creates an empty list
{
  LengthUsed = 0;
  LengthAllocated = 10;

  List = (int *)calloc(LengthAllocated, sizeof(int));
  if (List==NULL) {
    fprintf(stderr,"IntList::Intlist() memory alloc of %d failed\n", LengthAllocated);
    exit(1);
  }
}

IntList::~IntList()		// Destructor - master of destruction
{
  free(List);
}

void IntList::AddItem( const int value ) // Add item to the end of the list
{
  if (LengthUsed == LengthAllocated) {
    IncreaseAllocation();
  }

  List[LengthUsed++] = value;
}


void IntList::IncreaseAllocation(void) // Allocate some more memory for the List
{
  LengthAllocated += 10;
  List = (int *)realloc(List, LengthAllocated * sizeof(int) );
  if (List==NULL) {
    fprintf(stderr,"IntList::IncreaseAllocation memory realloc to %d failed\n", LengthAllocated);
    exit(1);
  }
}

void IntList::RemoveItem(const int n)	// Remove the nth item
{
  assert(((n >= 0)&&(n < LengthUsed)));

  for (int i=n;i<(LengthUsed-1);i++) {
    List[i] = List[i+1];
  }
  LengthUsed--;
}

void IntList::Remove(int n)	// Remove all items with value n
{
  bool done = false;

  while (!done) {
    done = true;

    for (int i=0;i<LengthUsed;i++) {
      if (List[i] == n) {
	RemoveItem(i);
	done = false;
	break;
      }
    }
  }
}

void IntList::Clear()
{
  LengthUsed = 0;
}

IntList &IntList::operator=(IntList &what)
{
  if (this != &what) {
    this->Clear();
    for (int i=0;i<what.Length();i++) {
      this->AddItem(what.Item(i));
    }
  }
  return *this;
}
