/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
// Smartboard includes
#include <util/Vec2.h>		// 2-D vectors
#include <util/Multiline.h>	// multilines
#include <util/Box2.h>		// 2-D boxes.

// Tcl/Tk includes
#include <tcl.h>
#include <tk.h>

#include <cassert>
#include <cstdlib>
#include <cstdio>
#include <cstring>
#include <iostream>

#include "common.h"
#include "rec_data.h"
#include "intlist.h"
//#include "mutex.h"

#define DEB 0
#define SHOWLOCKING 0

// Constructor
RecData::RecData()
{
  StrokeCount = 0;
  CollectionCount = 0;
  CharacterCount = 0;
  NextStID = 0;
  NextCoID = 0;
  NextChID = 0;
}

// //////////////////////////////////////////////////////////////////////////////
// Spit out everything we know!
void RecData::Display_All_Rec_Data(void)
{
  int i;


  printf("==============================================================================\n");
  printf("Strokes:\n");
  for (i=0;i<StrokeCount;i++) {
    printf("ID#%d points=%d, length=%d\n",
	   StrokeData[i].StID,
	   StrokeData[i].Stroke.Points(),
	   (int)StrokeData[i].Stroke.Length());
  }
  printf("\n");
  printf("Collections:\n");
  for (i=0;i<CollectionCount;i++) {
    printf("ID#%d(%s) ", CollectionData[i].CoID, CollectionIsUsed(CollectionData[i].CoID)?"Used":"Unused");

    for (int j=0;j<CollectionData[i].Items.Length();j++) {
      printf("%d ",CollectionData[i].Items.Item(j));
    }
    printf("\n");
  }
    
  printf("\n");
  printf("Characters:\n");
  for (i=0;i<CharacterCount;i++) {
    printf("ID#%d Collection=%d WhatIAm=%s BBox={%d %d %d %d} otherchoices=%s\n",
	   CharacterData[i].ChID,
	   CharacterData[i].MyCollection,
	   CharacterData[i].WhatIAm,
	   (int)CharacterData[i].MyBoundingBox.Xmin(),
	   (int)CharacterData[i].MyBoundingBox.Ymin(),
	   (int)CharacterData[i].MyBoundingBox.Xmax(),
	   (int)CharacterData[i].MyBoundingBox.Ymax(),
	   CharacterData[i].OtherChoices);
  }
  printf("==============================================================================\n");

}

// int RecData::Display_All_Rec_Data(ClientData client_data,
// 			 Tcl_Interp* interp,
// 			 int argc,
// 			 char *argv[])
// {
//   Display_All_Rec_Data();
//   return TCL_OK;
// }

// //////////////////////////////////////////////////////////////////////////////


int RecData::Next_Unique_Stroke_ID()
{
  return NextStID++;
}

// int RecData::Next_Unique_Stroke_ID(ClientData client_data,
// 			  Tcl_Interp* interp,
// 			  int argc,
// 			  char *argv[])
// {
//   char res[1024];

//   sprintf(res, "%d", NextStID++);

//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

int RecData::Next_Unique_Collection_ID(void)
{
  int res;



  res = NextCoID++;


  return res;
}

// int RecData::Next_Unique_Collection_ID(ClientData client_data,
// 			      Tcl_Interp* interp,
// 			      int argc,
// 			      char *argv[])
// {
//   char res[1024];
//   sprintf(res, "%d", Next_Unique_Collection_ID());

//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

int RecData::Next_Unique_Character_ID(void)
{
  int res;

  res = NextChID++;

  return res;
}

// int RecData::Next_Unique_Character_ID(ClientData client_data,
// 			     Tcl_Interp* interp,
// 			     int argc,
// 			     char *argv[])
// {
//   char res[1024];
//   sprintf(res, "%d", Next_Unique_Character_ID());
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

// //////////////////////////////////////////////////////////////////////////////
int RecData::StrokeIDForNum(int n)
{
  return StrokeData[n].StID;
}

int RecData::NumberOfStrokes(void)
{
  return StrokeCount;
}

int RecData::StrokeNumForID(const int ID)
{
  int res = -1;
  
  for (int i=0;i<StrokeCount;i++) {
    if (StrokeData[i].StID == ID) {
      res = i;
      break;
    }
  }
  
  return res;
}

char *TranslateStrokeData(char *Strokes_In, const Vec2 &TBy)
{
  char *instring;
  char *NewData;
  char *s;
  int xval,yval;




  NewData = (char *)calloc(2*strlen(Strokes_In),sizeof(char));  // WOMBLEish!
  NewData[0] = '\0';
  instring = strdup(Strokes_In);
  s = strtok(instring," "); // split it up at spaces
  while (s!=NULL) { // NULL from strtok indicates that there is no more
    if (!strcmp(s,"jump")) { // there was a jump in the line
      //      sprintf(NewData,"%s jump jump",NewData);
      STRSPACECAT(NewData,"jump jump");

      s = strtok(NULL," "); // gobble up the other jump
      if (s==NULL) {
	fprintf(stderr,"Eh? Got NULL (1)\n");
	exit(1);
      }
    } else {
      xval = atoi(s);

      s = strtok(NULL," ");
      if (s==NULL) {
	fprintf(stderr,"Eh? Got NULL (2)\n");
	exit(1);
      }
      yval = atoi(s);
      //      sprintf(NewData,"%s %d %d",NewData, (int)(xval+TBy.X()), (int)(yval+TBy.Y()));
      char temp[255];
      sprintf(temp,"%d %d", (int)(xval+TBy.X()), (int)(yval+TBy.Y()));
      STRSPACECAT(NewData,temp);
    }
    s = strtok(NULL," ");
  } // end of strtok while loop


  free(instring);



  return(NewData);
}

Multiline *RecData::CharStrokeDataToMultiline(char *Strokes_In)
{
  char                *instring;
  Multiline            ML;
  int                  xval;
  int                  yval;
  char                *s;

  ML.Clear();

  instring = strdup(Strokes_In);
  s = strtok(instring," "); // split it up at spaces
  while (s!=NULL) { // NULL from strtok indicates that there is no more
    Vec2      P;

    if (!strcmp(s,"jump")) { // there was a jump in the line
      ML << Multiline::Jump;

      s = strtok(NULL," "); // gobble up the other jump
      if (s==NULL) {
	fprintf(stderr,"Eh? Got NULL (1)\n");
	exit(1);
      }
    } else {
      xval = atoi(s);

      s = strtok(NULL," ");
      if (s==NULL) {
	fprintf(stderr,"Eh? Got NULL (2)\n");
	exit(1);
      }
      yval = atoi(s);
      
      P.X() = xval;
      P.Y() = yval;

      ML << P;
    }
    s = strtok(NULL," ");
  } // end of strtok while loop
  
  Multiline *Ret;

  Ret = new Multiline;
  (*Ret) = ML;
  free(instring);
  return Ret;
}

// int RecData::AddStroke(ClientData client_data,
// 	      Tcl_Interp* interp,
// 	      int argc,
// 	      char *argv[])
int RecData::AddStroke(int ID, char *data)
{
//   int ID;
//   char *data;

//   if (argc!=3) {
//     sprintf(interp->result,"rec_data.cc:AddStroke - Incorrect number of arguments. (want data and ID)\n");
//     return TCL_ERROR;
//   }

//   ID = atoi(argv[2]);
//   data = argv[1];
  if (DEB) printf("rec_data.cc:AddStroke(<data>,%d)\n",ID);
  StrokeData[StrokeCount].Stroke = CharStrokeDataToMultiline(data);
  StrokeData[StrokeCount].CharData = strdup(data);
  StrokeData[StrokeCount].StID = ID;
  StrokeCount++;

  return TCL_OK;
}

// Return the data for a specified strokes
char *RecData::GetStrokeData(const int ID)
{
  int n;

  n = StrokeNumForID(ID);
  
  return StrokeData[n].CharData;
}

// int RecData::GetStrokeData(ClientData client_data,
// 		  Tcl_Interp* interp,
// 		  int argc,
// 		  char *argv[])
// {
//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:GetStrokeData - Incorrect number of arguments (want StrokeID).\n");
//     return TCL_ERROR;
//   }




//   Tcl_SetResult(interp, GetStrokeData(atoi(argv[1])), TCL_VOLATILE);



//   return TCL_OK;
// }

Multiline &RecData::GetStrokeMultiline(const int ID)
{
  int n;

  n = StrokeNumForID(ID);

  return StrokeData[n].Stroke;
}

void RecData::DeleteStroke(int StrokeNum)
{
  // WOMBLE  - delete multiline in StrokeData[i] and [last-i]

  for (int i=StrokeNum;i<(StrokeCount-1);i++) {
    StrokeData[i].Stroke = StrokeData[i+1].Stroke;
    StrokeData[i].StID=StrokeData[i+1].StID;
    if (StrokeData[i].CharData != NULL) {
      free(StrokeData[i].CharData);
    }
    StrokeData[i].CharData=strdup(StrokeData[i+1].CharData);   // WOMBLE!! Memory leak coming up! free before copying!
  }

  StrokeCount--;

}


// int RecData::DeleteStroke(ClientData client_data,
// 		 Tcl_Interp* interp,
// 		 int argc,
// 		 char *argv[])
// {
//   int ID;
//   int n;

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:DeleteStroke - Incorrect number of arguments (want ID).\n");
//     return TCL_ERROR;
//   }

//   ID = atoi(argv[1]);

//   n = StrokeNumForID(ID);

//   DeleteStroke(n);

//   return TCL_OK; 
// }

// //////////////////////////////////////////////////////////////////////////////

int RecData::CollNumForID(const int ID)
{
  int res = -1;
  
  for (int i=0;i<CollectionCount;i++) {
    if (CollectionData[i].CoID == ID) {
      res = i;
      break;
    }
  }
  
  return res;
}

// add stroke id ID to collection id CollID
void RecData::AddToCollection(int CollID, int ID)
{
  int CollNum;


  CollNum = CollNumForID(CollID);

  if (CollNum == -1) {
    EmptyCollection(CollID);
    CollNum = CollNumForID(CollID);
  }

  assert(CollNum != -1); // this shouldn't fail, of course. :)
  if (DEB) printf("add to collection ID#%d (c#%d) value %d\n",CollID,CollNum,ID);
  CollectionData[CollNum].Items.AddItem(ID);


}

// int RecData::AddToCollection(ClientData client_data,
// 		    Tcl_Interp* interp,
// 		    int argc,
// 		    char *argv[])
// {
//   int CollID;
//   int ID;

//   if (argc!=3) {
//     sprintf(interp->result,"rec_data.cc:AddToCollection - Incorrect number of arguments (want CollID,ID).\n");
//     return TCL_ERROR;
//   }


//   CollID = atoi(argv[1]);
//   ID = atoi(argv[2]);
  
//   AddToCollection(CollID, ID);
//   return TCL_OK;
// }

void RecData::RemoveCollection(int Number)
{
  for (int i=Number;i<CollectionCount-1;i++) {
    CollectionData[i].CoID = CollectionData[i+1].CoID;
    CollectionData[i].Items.Clear();
    for (int j=0;j<CollectionData[i+1].Items.Length();j++) {
      CollectionData[i].Items.AddItem(CollectionData[i+1].Items.Item(j));
    }
  }
  CollectionCount--;
}

// int RecData::RemoveCollection(ClientData client_data,
// 		     Tcl_Interp* interp,
// 		     int argc,
// 		     char *argv[])
// {
//   int CollNum;

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:RemoveCollection - Incorrect number of arguments (want CollID).\n");
//     return TCL_ERROR;
//   }
  
//   CollNum = CollNumForID(atoi(argv[1]));
//   assert(CollNum != -1);

//   RemoveCollection(CollNum);

//   return TCL_OK;
// }

// int RecData::CollectionExists(ClientData client_data,
// 		     Tcl_Interp* interp,
// 		     int argc,
// 		     char *argv[])
// {
//   int CollID;
//   int CollNum;
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:CollectionExists - Incorrect number of arguments (want CollID).\n");
//     return TCL_ERROR;
//   }

//   CollID = atoi(argv[1]);

//   CollNum = CollNumForID(CollID);
//   sprintf(res, "%d", (CollNum==-1)?0:1);
//   Tcl_SetResult(interp, res, TCL_VOLATILE);

//   return TCL_OK;

// }

void RecData::RemoveFromCollection(int CollID, int ID)
{
  int CollNum = CollNumForID(CollID);




  CollectionData[CollNum].Items.Remove(ID);
  if (CollectionData[CollNum].Items.Length() == 0) {
    RemoveCollection(CollNum);
  }

}

// int RecData::RemoveFromCollection(ClientData client_data,
// 			 Tcl_Interp* interp,
// 			 int argc,
// 			 char *argv[])
// {
//   int CollID;
//   int ID;

//   if (argc!=3) {
//     sprintf(interp->result,"rec_data.cc:RemoveFromCollection - Incorrect number of arguments (want CollID,ID).\n");
//     return TCL_ERROR;
//   }

//   CollID = atoi(argv[1]);
//   ID = atoi(argv[2]);
//   RemoveFromCollection(CollID, ID);
//   return TCL_OK;
// }

void RecData::EmptyCollection(int CollID)
{
  int CollNum;

  CollNum = CollNumForID(CollID);

  if (CollNum == -1) {
    // This collection ID is new
    CollNum = CollectionCount++;

    CollectionData[CollNum].CoID = CollID;
  }
  CollectionData[CollNum].Items.Clear();
}

// int RecData::EmptyCollection(ClientData client_data,
// 		    Tcl_Interp* interp,
// 		    int argc,
// 		    char *argv[])
// {
//   int CollID;

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:EmptyCollection - Incorrect number of arguments (want CollID).\n");
//     return TCL_ERROR;
//   }

//   CollID = atoi(argv[1]);

//   EmptyCollection(CollID);

//   return TCL_OK;

// }


// //////////////////////////////////////////////////////////////////////////////
int RecData::CharNumForID(const int ID)
{
  int res = -1;
  
  for (int i=0;i<CharacterCount;i++) {
    if (CharacterData[i].ChID == ID) {
      res = i;
      break;
    }
  }
  
  return res;
}



void RecData::SetCharacterMultiline(int CharID, Multiline &ML)
{
  int CharNum;



  CharNum = CharNumForID(CharID);

  if (CharNum == -1) {
    CharNum = CharacterCount++;
    CharacterData[CharNum].WhatIAm = NULL;
    CharacterData[CharNum].OtherChoices = NULL;
    CharacterData[CharNum].ChID = CharID;
  }

  CharacterData[CharNum].MyStrokes = ML;

}

void RecData::SetCharacterWhatItIs(int CharID, char *WhichWhat)
{
  int CharNum;


  CharNum = CharNumForID(CharID);

  if (CharNum == -1) {
    CharNum = CharacterCount++;
    CharacterData[CharNum].WhatIAm = NULL;
    CharacterData[CharNum].OtherChoices = NULL;
    CharacterData[CharNum].ChID = CharID;
  }

  if (CharacterData[CharNum].WhatIAm != NULL) {
    free(CharacterData[CharNum].WhatIAm);
  }
  CharacterData[CharNum].WhatIAm = strdup(WhichWhat);

}

void RecData::SetCharacterOtherChoices(int CharID, char *WhichWhat)
{
  int CharNum;



  CharNum = CharNumForID(CharID);

  if (CharNum == -1) {
    CharNum = CharacterCount++;
    CharacterData[CharNum].WhatIAm = NULL;
    CharacterData[CharNum].OtherChoices = NULL;
    CharacterData[CharNum].ChID = CharID;
  }

  if (CharacterData[CharNum].OtherChoices != NULL) {
    free(CharacterData[CharNum].OtherChoices);
  }
  CharacterData[CharNum].OtherChoices = strdup(WhichWhat);

}

void RecData::SetCharacterCollection(int CharID, int MyCollID)
{


  int CharNum = CharNumForID(CharID);
  int MyCollNum = CollNumForID(MyCollID);

  if (CharNum == -1) {
    CharNum = CharacterCount++;
    CharacterData[CharNum].WhatIAm = NULL;
    CharacterData[CharNum].OtherChoices = NULL;
    CharacterData[CharNum].ChID = CharID;
  }


  CharacterData[CharNum].MyCollection = MyCollID;
  // Update bounding box here!
  CharacterData[CharNum].MyBoundingBox = Box2d::Null;
  for (int i=0;i<CollectionData[MyCollNum].Items.Length();i++) {
    // "Box1 << Box2" means expand Box1 to include Box2
    int StrokeID = CollectionData[MyCollNum].Items.Item(i); // which stroke ID
    int StrokeNum = StrokeNumForID(StrokeID);
    CharacterData[CharNum].MyBoundingBox << StrokeData[StrokeNum].Stroke.Box();
  } // for (int i...

}

// Set a particular character's info, creating if necessary
// int RecData::SetCharacter(ClientData client_data,
// 		 Tcl_Interp* interp,
// 		 int argc,
// 		 char *argv[])
// {
//   if (argc!=4) {
//     sprintf(interp->result,"rec_data.cc:SetCharacter - Incorrect number of arguments (want CharID, RecordItem, Value).\n");
//     return TCL_ERROR;
//   }
//   int CharID = atoi(argv[1]);

//   if (!strcmp(argv[2],"MyCollection")) {
//     int MyCollID = atoi(argv[3]);

//     SetCharacterCollection(CharID, MyCollID);
//   } else {
//     if (!strcmp(argv[2],"WhatIAm")) {
//       SetCharacterWhatItIs(CharID, argv[3]);
//     } else {
//       if (!strcmp(argv[2],"OtherChoices")) {
// 	SetCharacterOtherChoices(CharID, argv[3]);
//       } else {
// 	sprintf(interp->result,"rec_data.cc:SetCharacter - Bad field name \"%s\"\n",argv[2]);
// 	return TCL_ERROR;
//       }
//     }
//   }

//   return TCL_OK;
// }


void RecData::RemoveCharacter(int CharNum)
{


  for (int i=CharNum;i<CharacterCount-1;i++) {
    CharacterData[i].MyCollection = CharacterData[i+1].MyCollection;
    CharacterData[i].ChID = CharacterData[i+1].ChID;
    if (CharacterData[i].WhatIAm != NULL) {
      free(CharacterData[i].WhatIAm);
    }
    CharacterData[i].WhatIAm = strdup(CharacterData[i+1].WhatIAm); // WOMBLE!! Memory leak coming up!
    CharacterData[i].MyBoundingBox = CharacterData[i+1].MyBoundingBox;
    CharacterData[i].MyStrokes = CharacterData[i+1].MyStrokes;
    if (CharacterData[i].OtherChoices != NULL) {
      free(CharacterData[i].OtherChoices);
    }
    CharacterData[i].OtherChoices = strdup(CharacterData[i+1].OtherChoices); // WOMBLE!! Memory leak coming up!
  }
  CharacterCount--;

}

// Remove a given character
// int RecData::RemoveCharacter(ClientData client_data,
// 		    Tcl_Interp* interp,
// 		    int argc,
// 		    char *argv[])
// {
//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:RemoveCharacter - Incorrect number of arguments (want CharID).\n");
//     return TCL_ERROR;
//   }
//   int CharNum = CharNumForID(atoi(argv[1]));

//   assert(CharNum != -1);

//   RemoveCharacter(CharNum);

//   return TCL_OK;
// }

Character *RecData::GetCharacterData(int CharID)
{
  int CharNum = CharNumForID(CharID);
  assert(CharNum != -1);
    
  return &(CharacterData[CharNum]);
}

// Get a character's info
// int RecData::GetCharacter(ClientData client_data,
// 		 Tcl_Interp* interp,
// 		 int argc,
// 		 char *argv[])
// {
//   char StrokeDataString[8192];
//   char MyResult[16384];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:GetCharacter - Incorrect number of arguments (want CharID).\n");
//     return TCL_ERROR;
//   }




//   int CharNum = CharNumForID(atoi(argv[1]));

//   assert(CharNum != -1);

//   int MyCollNum = CollNumForID(CharacterData[CharNum].MyCollection);

//   StrokeDataString[0] = '\0';

//   for (int i=0;i<CollectionData[MyCollNum].Items.Length();i++) {
//     int StrokeID = CollectionData[MyCollNum].Items.Item(i);
//     int StrokeNum = StrokeNumForID(StrokeID);

//     //sprintf(StrokeDataString,"%s %s",StrokeDataString, StrokeData[StrokeNum].CharData);
//     STRSPACECAT(StrokeDataString, StrokeData[StrokeNum].CharData);
    
//     if (i!=(CollectionData[MyCollNum].Items.Length() - 1)) {
//       //      sprintf(StrokeDataString,"%s jump jump",StrokeDataString);
//       STRSPACECAT(StrokeDataString,"jump jump");
//     }
//   }
  

//   // ID Collection WhatIAm BoundingBox StrokeData OtherChoices
//   sprintf(MyResult,"%d %d %s {%d %d %d %d} {%s} {%s}",
// 	  CharacterData[CharNum].ChID,
// 	  CharacterData[CharNum].MyCollection,
// 	  CharacterData[CharNum].WhatIAm,
// 	  (int)CharacterData[CharNum].MyBoundingBox.Xmin(),
// 	  (int)CharacterData[CharNum].MyBoundingBox.Ymin(),
// 	  (int)CharacterData[CharNum].MyBoundingBox.Xmax(),
// 	  (int)CharacterData[CharNum].MyBoundingBox.Ymax(),
// 	  StrokeDataString,
// 	  CharacterData[CharNum].OtherChoices);

//   Tcl_SetResult(interp, MyResult, TCL_VOLATILE);

//   return TCL_OK;
// }

int RecData::CharIDForNum(int CharNum)
{
  return CharacterData[CharNum].ChID;
}

// int RecData::CharIDForNum(ClientData client_data,
// 		 Tcl_Interp* interp,
// 		 int argc,
// 		 char *argv[])
// {
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:CharIDForNum - Incorrect number of arguments (want CharNum).\n");
//     return TCL_ERROR;
//   }
  

//   sprintf(res,"%d",CharIDForNum(atoi(argv[1])));
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

int RecData::NumberOfCharacters(void)
{
  return CharacterCount;
}

// int RecData::NumberOfCharacters(ClientData client_data,
// 		       Tcl_Interp* interp,
// 		       int argc,
// 		       char *argv[])
// {
//   char res[1024];

//   sprintf(res,"%d",NumberOfCharacters());
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

bool RecData::CollectionIsUsed(int CollID)
{
  for (int i=0;i<CharacterCount;i++) {
    if (CharacterData[i].MyCollection == CollID) {
      return true;
    }      
  }

  return false;
}


// int RecData::CollectionIsUsed(ClientData client_data,
// 		     Tcl_Interp* interp,
// 		     int argc,
// 		     char *argv[])
// {
//   int CollID;
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:CollectionIsUsed - Incorrect number of arguments (want CollID).\n");
//     return TCL_ERROR;
//   }

//   CollID = atoi(argv[1]);

//   sprintf(res,"%d",CollectionIsUsed(CollID)?1:0);
//   Tcl_SetResult(interp, res, TCL_VOLATILE);

//   return TCL_OK;
// }

int RecData::NumberOfCollections(void)
{
  return CollectionCount;
}

// int RecData::NumberOfCollections(ClientData client_data,
// 			Tcl_Interp* interp,
// 			int argc,
// 			char *argv[])
// {
//   char res[1024];

//   sprintf(res, "%d",NumberOfCollections());
//   Tcl_SetResult(interp, res, TCL_VOLATILE); 
//   return TCL_OK;
// }

int RecData::CollectionIDForNum(int n)
{
  return CollectionData[n].CoID;
}

// int RecData::CollectionIDForNum(ClientData client_data,
// 		       Tcl_Interp* interp,
// 		       int argc,
// 		       char *argv[])
// {
//   int CollNum;
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:CollectionIDForNum - Incorrect number of arguments (want CollID).\n");
//     return TCL_ERROR;
//   }

//   CollNum = atoi(argv[1]);

//   sprintf(res, "%d",CollectionIDForNum(CollNum));
//   Tcl_SetResult(interp, res, TCL_VOLATILE); 
//   return TCL_OK;
// }

int RecData::CollectionSize(int CollID)
{



  int CollNum = CollNumForID(CollID);

  if (CollNum == -1) {
    Display_All_Rec_Data();
    printf("collnum == -1 when trying to find CollID=%d\n",CollID);
  }
  assert(CollNum != -1);

  int ret = CollectionData[CollNum].Items.Length();

  return ret;
}

// int RecData::CollectionSize(ClientData client_data,
// 		   Tcl_Interp* interp,
// 		   int argc,
// 		   char *argv[])
// {
//   int CollID;
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:CollectionSize - Incorrect number of arguments (want CollID).\n");
//     return TCL_ERROR;
//   }

//   CollID = atoi(argv[1]);

//   sprintf(res, "%d", CollectionSize(CollID));
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

int RecData::CollectionItem(int CollID, int ItemNum)
{



  int CollNum = CollNumForID(CollID);

  assert(CollNum != -1);
  assert(ItemNum < CollectionData[CollNum].Items.Length());
  int ret = (CollectionData[CollNum].Items.Item(ItemNum));

  return ret;
}

// int RecData::CollectionItem(ClientData client_data,
// 		   Tcl_Interp* interp,
// 		   int argc,
// 		   char *argv[])
// {
//   int CollID;
//   int ItemNum;
//   char res[1024];

//   if (argc!=3) {
//     sprintf(interp->result,"rec_data.cc:CollectionItem - Incorrect number of arguments (want CollID, ItemNum).\n");
//     return TCL_ERROR;
//   }

//   CollID = atoi(argv[1]);
//   ItemNum = atoi(argv[2]);
//   sprintf(res,"%d",CollectionItem(CollID, ItemNum));
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }


// Find out if a given stroke ID is in a collection somewhere.
// argv[1] = stroke id
// return: -1 = nope
//     or:  Collection ID number it was found in
int RecData::StrokeIsInACollection(int StrokeID)
{


  for (int i=0;i<CollectionCount;i++) {
    for (int j=0;j<CollectionData[i].Items.Length();j++) {
      if (CollectionData[i].Items.Item(j) == StrokeID) {
	return (CollectionData[i].CoID);
      }
    }
  }

  return -1; // didn't find it
}

// int RecData::StrokeIsInACollection(ClientData client_data,
// 			  Tcl_Interp* interp,
// 			  int argc,
// 			  char *argv[])
// {
//   int StrokeID;
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:StrokeIsInACollection - Incorrect number of arguments (want StrokeID).\n");
//     return TCL_ERROR;
//   }

//   StrokeID = atoi(argv[1]);
  
//   sprintf(res,"%d",StrokeIsInACollection(StrokeID));
//   Tcl_SetResult(interp, res, TCL_VOLATILE);

//   return TCL_OK;
// }

// Find out if a given collection ID is in a character somewhere.
// argv[1] = collection id
// return: -1 = nope
//     or:  Character ID number it was found in
// int RecData::CollectionIsInACharacter(ClientData client_data,
// 			     Tcl_Interp* interp,
// 			     int argc,
// 			     char *argv[])
// {
//   int LookingFor;
//   char res[1024];

//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:CollectionIsInACharacter - Incorrect number of arguments (want CollectionID).\n");
//     return TCL_ERROR;
//   }

//   LookingFor = atoi(argv[1]);


//   for (int i=0;i<CharacterCount;i++) {
//     if (CharacterData[i].MyCollection == LookingFor) {
//       sprintf(res,"%d",CharacterData[i].ChID);
//       Tcl_SetResult(interp, res, TCL_VOLATILE);

//       return TCL_OK;
//     }
//   }

//   sprintf(res,"%d",-1);
//   Tcl_SetResult(interp, res, TCL_VOLATILE);

//   return TCL_OK;
// }


// int RecData::StrokeIDForNum(ClientData client_data,
// 		   Tcl_Interp* interp,
// 		   int argc,
// 		   char *argv[])
// {
//   char res[1024];
//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:StrokeIDForNum - Incorrect number of arguments (want Num).\n");
//     return TCL_ERROR;
//   }

//   sprintf(res,"%d",StrokeIDForNum(atoi(argv[1])));
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

// int RecData::NumberOfStrokes(ClientData client_data,
// 		    Tcl_Interp* interp,
// 		    int argc,
// 		    char *argv[])
// {
//   char res[1024];

//   sprintf(res,"%d",NumberOfStrokes());
//   Tcl_SetResult(interp, res, TCL_VOLATILE);
//   return TCL_OK;
// }

// Moving stuff around
// Translate a given stroke
void RecData::TranslateStroke(int StrokeID, const Vec2 &TBy)
{
  int StrokeNum = StrokeNumForID(StrokeID);

  if (DEB) printf("TStroke %d(%d) %d,%d\n",StrokeID,StrokeNum, (int)TBy.X(),(int)TBy.Y());

  // translate the stroke data
  char *NewSD = TranslateStrokeData(StrokeData[StrokeNum].CharData, TBy);
  free(StrokeData[StrokeNum].CharData);
  StrokeData[StrokeNum].CharData = strdup(NewSD);

  // translate the multiline
  StrokeData[StrokeNum].Stroke += TBy; // translate the multiline

  free(NewSD);
}

// Translate all the strokes in a given collection
void RecData::TranslateCollection(int CollID, const Vec2 &TBy)
{
  int CollNum = CollNumForID(CollID);

  if (DEB) printf("TCollection %d(%d) %d,%d\n",CollID,CollNum, (int)TBy.X(),(int)TBy.Y());

  for (int i=0;i<CollectionData[CollNum].Items.Length();i++) {
    int StrokeID = CollectionData[CollNum].Items.Item(i);
    
    TranslateStroke(StrokeID, TBy);
  }
}

// int RecData::TranslateColl(ClientData client_data,
// 		  Tcl_Interp* interp,
// 		  int argc,
// 		  char *argv[])
// {
//   if (argc!=4) {
//     sprintf(interp->result,"rec_data.cc:TranslateColl - Incorrect number of arguments (want CollID, DeltaX, DeltaY).\n");
//     return TCL_ERROR;
//   }

//   int CollID = atoi(argv[1]);
//   int DeltaX = atoi(argv[2]);
//   int DeltaY = atoi(argv[3]);
//   Vec2 TranslateBy(DeltaX,DeltaY);

//   TranslateCollection(CollID, TranslateBy);

//   return TCL_OK;
// }

// int RecData::TranslateStroke(ClientData client_data,
// 		    Tcl_Interp* interp,
// 		    int argc,
// 		    char *argv[])
// {
//   if (argc!=4) {
//     sprintf(interp->result,"rec_data.cc:TranslateStroke - Incorrect number of arguments (want StrokeID, DeltaX, DeltaY).\n");
//     return TCL_ERROR;
//   }

//   int StrokeID = atoi(argv[1]);
//   int DeltaX = atoi(argv[2]);
//   int DeltaY = atoi(argv[3]);

//   Vec2 TranslateBy(DeltaX,DeltaY);


//   TranslateStroke(StrokeID, TranslateBy);

//   return TCL_OK;
// }

void RecData::TranslateChar(int CharID, int DeltaX, int DeltaY)
{

  Vec2 TranslateBy(DeltaX,DeltaY);

  int CharNum = CharNumForID(CharID);

  if (DEB) printf("TChar %d(%d) %d,%d\n",CharID,CharNum, (int)TranslateBy.X(),(int)TranslateBy.Y());


  TranslateCollection(CharacterData[CharNum].MyCollection, TranslateBy);

  // Translate MyStrokes

  CharacterData[CharNum].MyStrokes += TranslateBy;
}

// int RecData::TranslateChar(ClientData client_data,
// 		  Tcl_Interp* interp,
// 		  int argc,
// 		  char *argv[])
// {
//   if (argc!=4) {
//     sprintf(interp->result,"rec_data.cc:TranslateChar - Incorrect number of arguments (want CharID, DeltaX, DeltaY).\n");
//     return TCL_ERROR;
//   }
//   int CharID = atoi(argv[1]);
//   int DeltaX = atoi(argv[2]);
//   int DeltaY = atoi(argv[3]);

//   TranslateChar(CharID, DeltaX, DeltaY);

//   return TCL_OK;
// }


// int RecData::Save_Rec_Data(ClientData client_data,
// 		  Tcl_Interp* interp,
// 		  int argc,
// 		  char *argv[])
// {
//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:Save_Rec_Data - Incorrect number of arguments (want Filename).\n");
//     return TCL_ERROR;
//   }

//   char *Filename = argv[1];

//   return Save_Rec_Data(Filename);
// }

int RecData::Save_Rec_Data(char *Filename, Tcl_Interp *interp)
{
  FILE *fh;
  int i;

  fh = fopen(Filename,"wt");

  if (fh==NULL) {
    sprintf(interp->result,"rec_data.cc:Save_Rec_Data - Error opening output file %s\n",Filename);

    return TCL_ERROR;
  } 

  fprintf(fh,"FFES Data file.  Version 1.\n");

  // Unique IDs
  fprintf(fh,"NextID\n");
  fprintf(fh,"%d\n%d\n%d\n",NextStID,NextCoID,NextChID);

  // Strokes
  fprintf(fh,"StrokeData\n");
  fprintf(fh,"%d\n",StrokeCount);
  for (i=0;i<StrokeCount;i++) {
    fprintf(fh,"%d\n%s\n",StrokeData[i].StID,StrokeData[i].CharData);
    // need to rebuild SD[i].Stroke on loading
  }
  // Collections
  fprintf(fh,"CollectionData\n");
  fprintf(fh,"%d\n",CollectionCount);
  for (i=0;i<CollectionCount;i++) {
    int j;

    fprintf(fh,"%d\n%d\n",CollectionData[i].CoID,CollectionData[i].Used?1:0);
    fprintf(fh,"%d\n",CollectionData[i].Items.Length());
    for (j=0;j<CollectionData[i].Items.Length();j++) {
      fprintf(fh,"%d\n",CollectionData[i].Items.Item(j));
    }
  }
  // CharacterData
  fprintf(fh,"CharacterData\n");
  fprintf(fh,"%d\n",CharacterCount);
  for (i=0;i<CharacterCount;i++) {
    fprintf(fh,"%d\n%d\n%s\n",
	    CharacterData[i].ChID,
	    CharacterData[i].MyCollection,
	    CharacterData[i].WhatIAm);
    fprintf(fh,"%g\n%g\n%g\n%g\n",
	    CharacterData[i].MyBoundingBox.Xmin(),
	    CharacterData[i].MyBoundingBox.Ymin(),
	    CharacterData[i].MyBoundingBox.Xmax(),
	    CharacterData[i].MyBoundingBox.Ymax());
    // need to rebuild MyStrokes on loading
    fprintf(fh,"%s\n",CharacterData[i].OtherChoices);
  }
  fclose(fh);

  return TCL_OK;
}

// int RecData::Load_Rec_Data(ClientData client_data,
// 		  Tcl_Interp* interp,
// 		  int argc,
// 		  char *argv[])
// {
//   if (argc!=2) {
//     sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Incorrect number of arguments (want Filename).\n");
//     return TCL_ERROR;
//   }


//   char *Filename = argv[1];

//   Load_Rec_Data(Filename, interp);
// }

int RecData::Load_Rec_Data(char *Filename, 
			   Tcl_Interp *interp)
{
  FILE *fh;
  int i;

  fh = fopen(Filename,"rt");

  if (fh==NULL) {
    sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Error opening input file %s\n",Filename);

    return TCL_ERROR;
  } 

  char InLine[4096];
  
  fgets(InLine,4096,fh);

  if (strcmp(InLine,"FFES Data file.  Version 1.\n") != 0) {
    sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Didn't get expected version in data file\n");

    return TCL_ERROR;
  }
  // Unique IDs
  fgets(InLine,4096,fh);
  if (strcmp(InLine,"NextID\n") != 0) {
    sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Didn't hit StrokeData section\n");

    return TCL_ERROR;
  }
  fgets(InLine,4096,fh);
  sscanf(InLine,"%d",&NextStID);
  fgets(InLine,4096,fh);
  sscanf(InLine,"%d",&NextCoID);
  fgets(InLine,4096,fh);
  sscanf(InLine,"%d",&NextChID);


  // Strokes
  fgets(InLine,4096,fh);
  if (strcmp(InLine,"StrokeData\n") != 0) {
    sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Didn't hit StrokeData section\n");

    return TCL_ERROR;
  }

  fgets(InLine,4096,fh);
  sscanf(InLine,"%d",&StrokeCount);
  for (i=0;i<StrokeCount;i++) {
    fgets(InLine,4096,fh);
    sscanf(InLine,"%d",&StrokeData[i].StID);
    fgets(InLine,4096,fh);
    // Remove trailing \n
    InLine[strlen(InLine)-1] = '\0';
    StrokeData[i].CharData = strdup(InLine);
    // need to rebuild SD[i].Stroke on loading:
    StrokeData[i].Stroke = CharStrokeDataToMultiline(StrokeData[i].CharData);
  }

  // Collections
  fgets(InLine,4096,fh);
  if (strcmp(InLine,"CollectionData\n") != 0) {
    sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Didn't hit CollectionData section\n");

    return TCL_ERROR;
  }

  fgets(InLine,4096,fh);
  sscanf(InLine,"%d",&CollectionCount);

  for (i=0;i<CollectionCount;i++) {
    int j,k;
    int temp;

    fgets(InLine,4096,fh);
    sscanf(InLine,"%d",&CollectionData[i].CoID);

    fgets(InLine,4096,fh);
    sscanf(InLine,"%d",&temp);
    CollectionData[i].Used = temp?true:false;

    CollectionData[i].Items.Clear();

    fgets(InLine,4096,fh);
    sscanf(InLine,"%d",&k);

    for (j=0;j<k;j++) {
      fgets(InLine,4096,fh);
      sscanf(InLine,"%d",&temp);
      CollectionData[i].Items.AddItem(temp);
    }

    assert(CollectionData[i].Items.Length() == k);
  }
  // CharacterData
  fgets(InLine,4096,fh);
  if (strcmp(InLine,"CharacterData\n") != 0) {
    sprintf(interp->result,"rec_data.cc:Load_Rec_Data - Didn't hit CharacterData section\n");

    return TCL_ERROR;
  }

  fgets(InLine,4096,fh);
  sscanf(InLine,"%d",&CharacterCount);

  for (i=0;i<CharacterCount;i++) {
    double j,k,l,m;

    fgets(InLine,4096,fh);
    sscanf(InLine,"%d",&CharacterData[i].ChID);

    fgets(InLine,4096,fh);
    sscanf(InLine,"%d",&CharacterData[i].MyCollection);
    fgets(InLine,4096,fh);
    // Remove trailing \n
    InLine[strlen(InLine)-1] = '\0';
    
    CharacterData[i].WhatIAm = strdup(InLine);

    fgets(InLine,4096,fh);
    sscanf(InLine,"%lf",&j);
    fgets(InLine,4096,fh);
    sscanf(InLine,"%lf",&k);
    fgets(InLine,4096,fh);
    sscanf(InLine,"%lf",&l);
    fgets(InLine,4096,fh);
    sscanf(InLine,"%lf",&m);
    CharacterData[i].MyBoundingBox = Box2d(Vec2(j,k),Vec2(l,m));
    // need to rebuild MyStrokes on loading
    //  MyCollection->CollectionData->Items->StrokeData->Stroke
    int CollID = CharacterData[i].MyCollection;
    Multiline ML;
    ML.Clear();
    for (int n=0;n<CollectionSize(CollID);n++) {
      Multiline MLTA;
      MLTA = GetStrokeMultiline( CollectionItem(CollID, n) ); // CollectionData[CollNumForID(MyCollection)].Items.Item(n)); 
      ML += MLTA;
    }
    CharacterData[i].MyStrokes = ML;
    fgets(InLine,4096,fh);
    // Remove trailing \n
    InLine[strlen(InLine)-1] = '\0';

    CharacterData[i].OtherChoices = strdup(InLine);
  }
  fclose(fh);

  return TCL_OK;
}



int RecData::Forget_Everything(void)
{
  
  while (CharacterCount > 0) {
    RemoveCharacter(0);
  }

  while (CollectionCount > 0) {
    RemoveCollection(0);
  }

  while (StrokeCount > 0) {
    DeleteStroke(0);
  }

  return TCL_OK;
}

RecData& RecData::operator=(RecData &what)
{
  int i;
  // check for self assignment
  if (this != &what) {
    // copy stuff from A to B.

    this->StrokeCount = what.StrokeCount;
    for (i=0;i<what.StrokeCount;i++) {
      this->StrokeData[i].Stroke = what.StrokeData[i].Stroke;
      this->StrokeData[i].StID = what.StrokeData[i].StID;
      this->StrokeData[i].CharData = strdup(what.StrokeData[i].CharData); // womble: memory leak!
    }

    this->CollectionCount = what.CollectionCount;
    for (i=0;i<what.CollectionCount;i++) {
      this->CollectionData[i].Items = what.CollectionData[i].Items;
      this->CollectionData[i].CoID = what.CollectionData[i].CoID;
      this->CollectionData[i].Used = what.CollectionData[i].Used;
    }
    this->CharacterCount = what.CharacterCount;
    for (i=0;i<what.CharacterCount;i++) {
      this->CharacterData[i].MyCollection = what.CharacterData[i].MyCollection;
      this->CharacterData[i].ChID = what.CharacterData[i].ChID ;
      this->CharacterData[i].WhatIAm = strdup(what.CharacterData[i].WhatIAm); // womble: memory leak
      this->CharacterData[i].MyBoundingBox = what.CharacterData[i].MyBoundingBox;
      this->CharacterData[i].MyStrokes = what.CharacterData[i].MyStrokes;
      this->CharacterData[i].OtherChoices = strdup(what.CharacterData[i].OtherChoices); // womble: memory leak
    }
  }

  this->NextStID = what.NextStID;
  this->NextCoID = what.NextCoID;
  this->NextChID = what.NextChID;

  return *this;
}

