/* *************************************************************************
* Copyright (c) 1999, Steve Smithies                                       *
*                                                                          *
* This program is free software; you can redistribute it and/or modify it  *
* under the terms of the GNU General Public License as published by the    *
* Free Software Foundation.  See http://www.fsf.org/copyleft/gpl.html      *
*                                                                          *
* This program is distributed in the hope that it will be useful, but      *
* WITHOUT EXPRESS OR IMPLIED WARRANTY of merchantability or fitness for    *
* any particular purpose.  See the GNU General Public License for more     *
* details.                                                                 *
*                                                                          *
***************************************************************************/
#ifndef _REC_DATA_H_
#define _REC_DATA_H

#include "intlist.h"

#define MAXIMUM_STROKES 1024


struct STROKES_T {
  Multiline  Stroke;
  int        StID;
  char      *CharData;
};

struct STROKE_COLLECT_T {
  IntList    Items;
  int        CoID;
  bool       Used;
};

struct CHARACTER_T {
  int        MyCollection;	// the collection that this character is made up from
  int        ChID;		// My unique ID
  char      *WhatIAm;		// what this character has been recognised as
  Box2d      MyBoundingBox;	// Bounding box for this character.. Calculated on notification of MyCollection
  Multiline  MyStrokes;         // Strokes that make up this character.  Same info as can be gleaned
				// from MyCollection->CollectionData->Items->StrokeData->Stroke
  char      *OtherChoices;
};

typedef struct STROKES_T Strokes;
typedef struct STROKE_COLLECT_T StrokeCollection;
typedef struct CHARACTER_T Character;


class RecData {

public:
  // Variables
  Strokes StrokeData[MAXIMUM_STROKES];
  int StrokeCount;
  StrokeCollection CollectionData[MAXIMUM_STROKES];
  int CollectionCount;
  Character  CharacterData[MAXIMUM_STROKES];
  int CharacterCount;
  int NextStID;
  int NextCoID;
  int NextChID;
  

  // Constructor
  RecData();

  // Functions
  int Next_Unique_Stroke_ID();
  int Next_Unique_Collection_ID(void);
  int Next_Unique_Character_ID(void);

  // Strokes
  int AddStroke(int ID, char *data);
  int StrokeIDForNum(int n);
  char *GetStrokeData(const int ID);
  Multiline &GetStrokeMultiline(const int ID);
  int NumberOfStrokes(void);
  // Collections
  void AddToCollection(int CollID, int ID);
  void RemoveFromCollection(int CollID, int ID);
  void EmptyCollection(int CollID);
  int StrokeIsInACollection(int StrokeID);
  bool CollectionIsUsed(int CollID);
  int NumberOfCollections(void);
  int CollectionIDForNum(int n);
  int CollectionSize(int CollID);
  int CollectionItem(int CollID, int ItemNum);

  // Characters
  void SetCharacterMultiline(int CharID, Multiline &ML);
  Character *GetCharacterData(int CharID);
  int NumberOfCharacters(void);
  int CharIDForNum(int CharNum);
  void SetCharacterWhatItIs(int CharID, char *WhichWhat);
  void SetCharacterCollection(int CharID, int MyCollID);
  void SetCharacterOtherChoices(int CharID, char *WhichWhat);

  // Moving stuff around
  void TranslateChar(int CharID, int DeltaX, int DeltaY);
  // Load/save
  int Save_Rec_Data(char *Filename, Tcl_Interp *interp);
  int Load_Rec_Data(char *Filename, Tcl_Interp *interp);

  // Blah
  Multiline *CharStrokeDataToMultiline(char *Strokes_In);
  void Display_All_Rec_Data(void);
  int  Forget_Everything(void);
  int  StrokeNumForID(const int ID);
  void DeleteStroke(int StrokeNum);
  int  CollNumForID(const int ID);
  void RemoveCollection(int Number);
  int  CharNumForID(const int ID);
  void RemoveCharacter(int CharNum);
  void TranslateStroke(int StrokeID, const Vec2 &TBy);
  void TranslateCollection(int CollID, const Vec2 &TBy);

  // Operators
  RecData& operator=(RecData &);

};  // end of class RecData

#endif
